extends CharacterBody2D


const SPEED = 300.0
const JUMP_VELOCITY = -400.0
const BUMP_FORCE = 50
const PUSHFORCE = 750
@onready var right_ray: RayCast2D = $RightRay
@onready var left_ray: RayCast2D = $LeftRay

var faceLeft = false

var pushTarget
var pushLefEnabled = false
var pushRightEnabled = false
@onready var right_target: Node2D = $RightTarget
@onready var left_target: Node2D = $LeftTarget


func _physics_process(delta: float) -> void:
	# Add the gravity.
	if not is_on_floor():
		velocity += get_gravity() * delta

	# Handle jump.
	if Input.is_action_just_pressed("ui_accept") and is_on_floor():
		velocity.y = JUMP_VELOCITY

	# Get the input direction and handle the movement/deceleration.
	# As good practice, you should replace UI actions with custom gameplay actions.
	var direction := Input.get_axis("ui_left", "ui_right")
	if Input.is_action_just_pressed("shove") && pushRightEnabled && faceLeft == false:
		print("Shove a box")
		pushTarget.apply_central_impulse(Vector2(1,0) * PUSHFORCE * 2)
		pushRightEnabled = false
		
	if Input.is_action_just_pressed("shove") && faceLeft == true && pushLefEnabled == true:
		pushTarget.apply_central_impulse(Vector2(-1,0) * PUSHFORCE * 2)
		pushLefEnabled = false
		
	if Input.is_action_just_pressed("shoot"):
		print("I'm an owl: Shoot shoot")
		var myBullet = %SceneManager.bulletFactory()
		if not faceLeft:
			myBullet.transform = right_target.global_transform
			myBullet.setSpeed(750)
		if faceLeft == true:
			myBullet.transform = left_target.global_transform
			myBullet.setSpeed(-750)
		
	if direction:
		velocity.x = direction * SPEED
		if direction > 0: faceLeft = false
		if direction < 0: faceLeft = true
	else:
		velocity.x = move_toward(velocity.x, 0, SPEED)

	move_and_slide()
	if right_ray.is_colliding():
		print("Something is on my right")
		if not faceLeft:
			var collider = right_ray.get_collider()
			if collider is Node:
				if collider.is_in_group("box"):
					print("This is a pushable box")
					pushTarget = collider
					pushRightEnabled = true
	else:
		pushRightEnabled = false
		
	if left_ray.is_colliding():
		print("Something is on my left")
		if faceLeft:
			var collider = left_ray.get_collider()
			if collider is Node:
				if collider.is_in_group("box"):
					print("I can shove this box")
					pushTarget = collider
					pushLefEnabled = true
	else:
		pushLefEnabled = false
		
	for i in get_slide_collision_count():
		var c = get_slide_collision(i)
		if c.get_collider() is RigidBody2D:
			c.get_collider().apply_central_impulse(-c.get_normal() * BUMP_FORCE)
